from typing import Dict

from django.http import HttpRequest

from .decorators import render_html


@render_html()
def html_view(request: HttpRequest, template_name: str = None, context: Dict = None):
    if context is None:
        context = {}

    if template_name is not None:
        context["TEMPLATE"] = template_name

    return context



class RequestMethodMiddleware:
    """
    Adds the request method as boolean properties to the request object.

    Example: You can check that a request is a post with `request.is_post` instead
    of using the string equality `request.method == "POST"` which is more error prone.
    """

    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        setattr(request, "is_post", request.method == "POST")
        setattr(request, "is_get", request.method == "GET")
        setattr(request, "is_patch", request.method == "PATCH")
        setattr(request, "is_head", request.method == "HEAD")
        setattr(request, "is_put", request.method == "PUT")
        setattr(request, "is_delete", request.method == "DELETE")
        setattr(request, "is_connect", request.method == "CONNECT")
        setattr(request, "is_trace", request.method == "TRACE")

        response = self.get_response(request)

        return response



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['fbv']

package_data = \
{'': ['*']}

install_requires = \
['django>2.2.0']

extras_require = \
{'docs': ['Sphinx>=4.3.2,<5.0.0',
          'linkify-it-py>=1.0.3,<2.0.0',
          'myst-parser>=0.16.1,<0.17.0',
          'furo>=2021.11.23,<2022.0.0',
          'sphinx-copybutton>=0.4.0,<0.5.0',
          'toml']}

setup_kwargs = {
    'name': 'django-fbv',
    'version': '0.2.0',
    'description': 'Utilities to make function-based views cleaner, more efficient, and better tasting. ',
    'long_description': '<p align="center">\n  <a href="https://django-fbv.readthedocs.io"><h1 align="center">django-fbv</h1></a>\n</p>\n<p align="center">Utilities to make Django function-based views cleaner, more efficient, and better tasting. 💥</p>\n\n![PyPI](https://img.shields.io/pypi/v/django-fbv?color=blue&style=flat-square)\n![PyPI - Downloads](https://img.shields.io/pypi/dm/django-fbv?color=blue&style=flat-square)\n![GitHub Sponsors](https://img.shields.io/github/sponsors/adamghill?color=blue&style=flat-square)\n\n📖 Complete documentation: https://django-fbv.readthedocs.io\n\n📦 Package located at https://pypi.org/project/django-fbv/\n\n## Features\n\n### decorators\n\n- [`fbv.decorators.render_html`](https://django-fbv.readthedocs.io/en/latest/decorators/#render-html): convenience decorator to specify the specific template that a view function should use\n- [`fbv.decorators.render_view`](https://django-fbv.readthedocs.io/en/latest/decorators/#render-view): convenience decorator to specify the specific template and content type that a view function should use\n\n### views\n\n- [`fbv.views.html_view`](https://django-fbv.readthedocs.io/en/latest/views/#html-view): directly render a template from `urls.py`\n\n### middleware\n\n- [`fbv.middleware.RequestMethodMiddleware`](https://django-fbv.readthedocs.io/en/latest/middleware/): adds a boolean property to the `request` for the current request\'s HTTP method\n\nRead all of the documentation at https://django-fbv.readthedocs.io/.\n',
    'author': 'adamghill',
    'author_email': 'adam@adamghill.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/adamghill/django-fbv/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>3.7,<4.0',
}


setup(**setup_kwargs)



import json
from functools import wraps
from os.path import join
from typing import Tuple

from django.core import serializers
from django.db import models
from django.http import JsonResponse
from django.shortcuts import render

__all__ = [
    "render_html",
    "render_view",
    "render_json",
]


DEFAULT_JSON_SEPARATORS = (", ", ": ")
MINIFIED_JSON_SEPARATORS = (",", ":")


def render_html(template_name: str = None):
    return render_view(template_name, content_type=None)


def render_view(template_name: str = None, content_type: str = None):
    """
    Decorator for Django function-based views that renders the passed-in template
    with the returned dictionary.

    Template name can be decorator parameter or `TEMPLATE` key in returned
    dictionary.  If view doesn't return dict then decorator simply returns output.

    Parameters:
     - template_name: template name to use
     - content_type: content type to send in response headers
    """

    def renderer(function):
        @wraps(function)
        def wrapper(request, *args, **kwargs):
            context = function(request, *args, **kwargs)

            if not isinstance(context, dict):
                return context

            _template_name = context.pop("TEMPLATE", template_name)

            if _template_name is None:
                module_name = function.__module__
                template_dir = module_name

                module_names = module_name.split(".")

                if len(module_names) > 1:
                    template_dir = join(*module_names[:-1])

                function_name = function.__name__
                _template_name = join(template_dir, f"{function_name}.html")

            return render(request, _template_name, context, content_type=content_type)

        return wrapper

    return renderer


def render_json(fields: Tuple[str] = None, separators: Tuple[str] = None):
    def renderer(function):
        @wraps(function)
        def wrapper(request, *args, fields=fields, separators=separators, **kwargs):
            context = function(request, *args, **kwargs)

            if isinstance(context, models.Model):
                context = json.loads(
                    serializers.serialize("json", [context], fields=fields)[1:-1]
                )
            elif isinstance(context, models.QuerySet):
                context = json.loads(
                    serializers.serialize("json", context, fields=fields)
                )
                context = {"models": context}
            else:
                assert (
                    fields is None
                ), "The `fields` kwarg should only be used when serializing Django models."

            if not isinstance(context, dict):
                return context

            if separators is None:
                separators = MINIFIED_JSON_SEPARATORS

            return JsonResponse(context, json_dumps_params={"separators": separators})

        return wrapper

    return renderer
